"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
/**
 * InfoBoxAction renders an action in an info window {@link InfoBox}
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent, MapMarkerDirective, InfoBoxComponent, InfoBoxActionDirective} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *    .map-container { height: 300px; }
 *  `],
 *  template: `
 *    <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-map-marker [Latitude]="lat" [Longitude]="lng" [Label]="'M'">
 *        <x-info-box>
 *          <x-info-box-action [Label]="actionlabel" (ActionClicked)="actionClicked(this)"></x-info-box-action>
 *        </x-info-box>
 *      </x-map-marker>
 *    </x-map>
 *  `
 * })
 * ```
 *
 * @export
 * @class InfoBoxActionDirective
 */
var InfoBoxActionDirective = /** @class */ (function () {
    function InfoBoxActionDirective() {
        /**
         * The label to display on the action
         *
         * @type {string}
         * @memberof InfoBoxActionDirective
         */
        /**
         * Emits an event when the action has been clicked
         *
         * @type {EventEmitter<void>}
         * @memberof InfoBoxActionDirective
         */
        this.ActionClicked = new core_1.EventEmitter();
    }
    InfoBoxActionDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: 'x-info-box-action'
                },] },
    ];
    /** @nocollapse */
    InfoBoxActionDirective.ctorParameters = function () { return []; };
    InfoBoxActionDirective.propDecorators = {
        'Label': [{ type: core_1.Input },],
        'ActionClicked': [{ type: core_1.Output },],
    };
    return InfoBoxActionDirective;
}());
exports.InfoBoxActionDirective = InfoBoxActionDirective;
//# sourceMappingURL=infobox-action.js.map