"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var bing_conversions_1 = require("../../services/bing/bing-conversions");
var canvas_overlay_1 = require("../canvas-overlay");
var bing_label_1 = require("./bing-label");
/**
 * Concrete implementing a canvas overlay to be placed on the map for Bing Maps.
 *
 * @export
 * @class BingCanvasOverlay
 */
var BingCanvasOverlay = /** @class */ (function (_super) {
    __extends(BingCanvasOverlay, _super);
    /**
     * Creates a new instance of the BingCanvasOverlay class.
     * @param  {HTMLCanvasElements => void} drawCallback A callback function that is triggered when the canvas is ready to be
     * rendered for the current map view.
     * @memberof BingCanvasOverlay
     * @constructor
     */
    function BingCanvasOverlay(drawCallback) {
        return _super.call(this, drawCallback) || this;
    }
    ///
    /// Public methods
    ///
    /**
     * Obtains geo coordinates for the click location
     *
     * @abstract
     * @param {Microsoft.Maps.IMouseEventArgs} e - The mouse event. Expected to implement {@link Microsoft.Maps.IMouseEventArgs}.
     * @returns {ILatLong} - {@link ILatLong} containing the geo coordinates of the clicked marker.
     * @memberof BingCanvasOverlay
     */
    BingCanvasOverlay.prototype.GetCoordinatesFromClick = function (e) {
        return { latitude: e.location.latitude, longitude: e.location.longitude };
    };
    ;
    /**
     * Gets the map associted with the label.
     *
     * @returns {Microsoft.Maps.Map}
     * @memberof BingCanvasOverlay
     * @method
     * @public
     */
    BingCanvasOverlay.prototype.GetMap = function () {
        return this.getMap();
    };
    /**
     * Returns a MapLabel instance for the current platform that can be used as a tooltip.
     * This method only generates the map label. Content and placement is the responsibility
     * of the caller. Note that this method returns null until OnLoad has been called.
     *
     * @returns {MapLabel} - The label to be used for the tooltip.
     * @memberof BingCanvasOverlay
     * @method
     * @public
     */
    BingCanvasOverlay.prototype.GetToolTipOverlay = function () {
        var o = {
            align: 'left',
            offset: new Microsoft.Maps.Point(0, 25),
            backgroundColor: 'bisque',
            hidden: true,
            fontSize: 12,
            fontColor: '#000000',
            strokeWeight: 0
        };
        var label = new bing_label_1.BingMapLabel(o);
        label.SetMap(this.GetMap());
        return label;
    };
    /**
     * CanvasOverlay loaded, attach map events for updating canvas.
     * @abstract
     * @method
     * @public
     * @memberof BingCanvasOverlay
     */
    BingCanvasOverlay.prototype.OnLoad = function () {
        var _this = this;
        var map = this.getMap();
        // Get the current map view information.
        this._zoomStart = map.getZoom();
        this._centerStart = map.getCenter();
        // Redraw the canvas.
        this.Redraw(true);
        // When the map moves, move the canvas accordingly.
        this._viewChangeEvent = Microsoft.Maps.Events.addHandler(map, 'viewchange', function (e) {
            if (map.getMapTypeId() === Microsoft.Maps.MapTypeId.streetside) {
                // Don't show the canvas if the map is in Streetside mode.
                _this._canvas.style.display = 'none';
            }
            else {
                // Re-drawing the canvas as it moves would be too slow. Instead, scale and translate canvas element.
                var zoomCurrent = map.getZoom();
                var centerCurrent = map.getCenter();
                // Calculate map scale based on zoom level difference.
                var scale = Math.pow(2, zoomCurrent - _this._zoomStart);
                // Calculate the scaled dimensions of the canvas.
                var newWidth = map.getWidth() * scale;
                var newHeight = map.getHeight() * scale;
                // Calculate offset of canvas based on zoom and center offsets.
                var pixelPoints = map.tryLocationToPixel([
                    bing_conversions_1.BingConversions.TranslateLocation(_this._centerStart),
                    centerCurrent
                ], Microsoft.Maps.PixelReference.control);
                var centerOffsetX = pixelPoints[1].x - pixelPoints[0].x;
                var centerOffsetY = pixelPoints[1].y - pixelPoints[0].y;
                var x = (-(newWidth - map.getWidth()) / 2) - centerOffsetX;
                var y = (-(newHeight - map.getHeight()) / 2) - centerOffsetY;
                // Update the canvas CSS position and dimensions.
                _this.UpdatePosition(x, y, newWidth, newHeight);
            }
        });
        // When the map stops moving, render new data on the canvas.
        this._viewChangeEndEvent = Microsoft.Maps.Events.addHandler(map, 'viewchangeend', function (e) {
            _this.UpdateCanvas();
        });
        // Update the position of the overlay when the map is resized.
        this._mapResizeEvent = Microsoft.Maps.Events.addHandler(map, 'mapresize', function (e) {
            _this.UpdateCanvas();
        });
        // set the overlay to ready state
        this._readyResolver(true);
    };
    /**
     * Sets the map for the label. Settings this to null remove the label from hte map.
     *
     * @param {Microsoft.Maps.Map} map - Map to associated with the label.
     * @memberof CanvasOverlay
     * @method
     * @public
     */
    BingCanvasOverlay.prototype.SetMap = function (map) {
        var m = this.GetMap();
        if (map === m) {
            return;
        }
        if (m) {
            m.layers.remove(this);
        }
        if (map != null) {
            map.layers.insert(this);
        }
    };
    ///
    /// Protected methods
    ///
    /**
     * Attaches the canvas to the map.
     * @memberof CanvasOverlay
     * @method
     * @public
     */
    BingCanvasOverlay.prototype.SetCanvasElement = function (el) {
        this.setHtmlElement(el);
    };
    /**
     * Remove the map event handlers.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    BingCanvasOverlay.prototype.RemoveEventHandlers = function () {
        // Remove all event handlers from the map.
        Microsoft.Maps.Events.removeHandler(this._viewChangeEvent);
        Microsoft.Maps.Events.removeHandler(this._viewChangeEndEvent);
        Microsoft.Maps.Events.removeHandler(this._mapResizeEvent);
    };
    /**
     * Updates the Canvas size based on the map size.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    BingCanvasOverlay.prototype.Resize = function () {
        var map = this.getMap();
        // Clear canvas by updating dimensions. This also ensures canvas stays the same size as the map.
        this._canvas.width = map.getWidth();
        this._canvas.height = map.getHeight();
    };
    /**
     * Updates the Canvas.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    BingCanvasOverlay.prototype.UpdateCanvas = function () {
        var map = this.getMap();
        // Only render the canvas if it isn't in streetside mode.
        if (map.getMapTypeId() !== Microsoft.Maps.MapTypeId.streetside) {
            this._canvas.style.display = '';
            // Reset CSS position and dimensions of canvas.
            this.UpdatePosition(0, 0, map.getWidth(), map.getHeight());
            // Redraw the canvas.
            this.Redraw(true);
            // Get the current map view information.
            this._zoomStart = map.getZoom();
            this._centerStart = map.getCenter();
        }
    };
    return BingCanvasOverlay;
}(canvas_overlay_1.CanvasOverlay));
exports.BingCanvasOverlay = BingCanvasOverlay;
/**
 * Helper function to extend the OverlayView into the CanvasOverlay
 *
 * @export
 * @method
 */
function MixinCanvasOverlay() {
    var x = BingCanvasOverlay.prototype;
    BingCanvasOverlay.prototype = new Microsoft.Maps.CustomOverlay();
    for (var y in x) {
        if (x[y] != null) {
            BingCanvasOverlay.prototype[y] = x[y];
        }
    }
    BingCanvasOverlay.prototype['onAdd'] = x['OnAdd'];
    BingCanvasOverlay.prototype['onLoad'] = x['OnLoad'];
    BingCanvasOverlay.prototype['onRemove'] = x['OnRemove'];
}
exports.MixinCanvasOverlay = MixinCanvasOverlay;
//# sourceMappingURL=bing-canvas-overlay.js.map