import { ILatLong } from '../../interfaces/ilatlong';
import { IInfoWindowOptions } from '../../interfaces/iinfo-window-options';
import { InfoWindow } from '../info-window';
/**
 * Concrete implementation of the {@link InfoWindow} contract for the Bing Maps V8 map architecture.
 *
 * @class BingInfoWindow
 * @implements {InfoWindow}
 * @export
 */
export declare class BingInfoWindow implements InfoWindow {
    private _infoBox;
    private _isOpen;
    /**
     * Gets whether the info box is currently open.
     *
     * @readonly
     * @type {boolean}
     * @memberof BingInfoWindow
     */
    readonly IsOpen: boolean;
    /**
     * Gets native primitve underlying the model.
     *
     * @type {Microsoft.Maps.Infobox}
     * @memberof BingInfoWindow
     * @property
     * @readonly
     * @public
     */
    readonly NativePrimitve: Microsoft.Maps.Infobox;
    /**
     * Creates an instance of BingInfoWindow.
     * @param {Microsoft.Maps.Infobox} _infoBox - A {@link Microsoft.Maps.Infobox} instance underlying the model
     * @memberof BingInfoWindow
     * @constructor
     * @public
     */
    constructor(_infoBox: Microsoft.Maps.Infobox);
    /**
     * Adds an event listener to the InfoWindow.
     *
     * @param {string} eventType - String containing the event for which to register the listener (e.g. "click")
     * @param {Function} fn - Delegate invoked when the event occurs.
     *
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    AddListener(eventType: string, fn: Function): void;
    /**
     * Closes the info window.
     *
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    Close(): void;
    /**
     * Gets the position of the info window.
     *
     * @returns {ILatLong} - Returns the geo coordinates of the info window.
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    GetPosition(): ILatLong;
    /**
     * Opens the info window.
     *
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    Open(): void;
    /**
     * Sets the info window options.
     *
     * @param {IInfoWindowOptions} options - Info window options to set. The options will be merged with any existing options.
     *
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    SetOptions(options: IInfoWindowOptions): void;
    /**
     * Sets the info window position.
     *
     * @param {ILatLong} position - Geo coordinates to move the anchor of the info window to.
     *
     * @memberof BingInfoWindow
     * @method
     * @public
     */
    SetPosition(position: ILatLong): void;
}
