"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var canvas_overlay_1 = require("../canvas-overlay");
var google_label_1 = require("./google-label");
/**
 * Concrete implementing a canvas overlay to be placed on the map for Google Maps.
 *
 * @export
 * @class GoogleCanvasOverlay
 */
var GoogleCanvasOverlay = /** @class */ (function (_super) {
    __extends(GoogleCanvasOverlay, _super);
    /**
     * Creates a new instance of the GoogleCanvasOverlay class.
     * @param  {HTMLCanvasElements => void} drawCallback A callback function that is triggered when the canvas is ready to be
     * rendered for the current map view.
     * @memberof GoogleCanvasOverlay
     * @constructor
     */
    function GoogleCanvasOverlay(drawCallback) {
        return _super.call(this, drawCallback) || this;
    }
    ///
    /// Public methods
    ///
    /**
     * Obtains geo coordinates for the click location
     *
     * @param {GoogleMapTypes.MouseEvent} e - The mouse event.
     * @returns {ILatLong} - {@link ILatLong} containing the geo coordinates of the clicked marker.
     * @memberof GoogleCanvasOverlay
     */
    GoogleCanvasOverlay.prototype.GetCoordinatesFromClick = function (e) {
        if (!e) {
            return null;
        }
        if (!e.latLng) {
            return null;
        }
        if (!e.latLng.lat || !e.latLng.lng) {
            return null;
        }
        return { latitude: e.latLng.lat(), longitude: e.latLng.lng() };
    };
    ;
    /**
     * Gets the map associted with the label.
     *
     * @returns {GoogleMapTypes.GoogleMap}
     * @memberof GoogleCanvasOverlay
     * @method
     * @public
     */
    GoogleCanvasOverlay.prototype.GetMap = function () {
        return this.getMap();
    };
    /**
     * Returns a MapLabel instance for the current platform that can be used as a tooltip.
     * This method only generates the map label. Content and placement is the responsibility
     * of the caller.
     *
     * @returns {MapLabel} - The label to be used for the tooltip.
     * @memberof GoogleCanvasOverlay
     * @method
     * @public
     */
    GoogleCanvasOverlay.prototype.GetToolTipOverlay = function () {
        var o = {
            align: 'left',
            offset: new google.maps.Point(0, 25),
            backgroundColor: 'bisque',
            hidden: true,
            fontSize: 12,
            fontColor: '#000000',
            strokeWeight: 0
        };
        o.zIndex = 100000;
        var label = new google_label_1.GoogleMapLabel(o);
        label.SetMap(this.GetMap());
        return label;
    };
    /**
     * Called when the custom overlay is added to the map. Triggers Onload....
     * @memberof GoogleCanvasOverlay
     */
    GoogleCanvasOverlay.prototype.OnAdd = function () {
        _super.prototype.OnAdd.call(this);
        this.OnLoad();
        this._canvas.style.zIndex = '100';
        // move the canvas above primitives such as polygons.
        // set the overlay to ready state
        this._readyResolver(true);
    };
    /**
     * Called whenever the canvas needs to be redrawn. This method does not do the actual
     * update, it simply scales the canvas. The actual redraw happens once the map is idle.
     * @memberof GoogleCanvasOverly
     * @method
     * @public
     */
    GoogleCanvasOverlay.prototype.OnDraw = function () {
        var isStreetView = false;
        var map = this.GetMap();
        if (isStreetView) {
            // Don't show the canvas if the map is in Streetside mode.
            this._canvas.style.display = 'none';
        }
        else {
            // Re-drawing the canvas as it moves would be too slow. Instead, scale and translate canvas element.
            // Upon idle or drag end, we can then redraw the canvas....
            var zoomCurrent = map.getZoom();
            var centerCurrent = map.getCenter();
            // Calculate map scale based on zoom level difference.
            var scale = Math.pow(2, zoomCurrent - this._zoomStart);
            // Calculate the scaled dimensions of the canvas.
            var el = map.getDiv();
            var w = el.offsetWidth;
            var h = el.offsetHeight;
            var newWidth = w * scale;
            var newHeight = h * scale;
            // Calculate offset of canvas based on zoom and center offsets.
            var projection = this.getProjection();
            var cc = projection.fromLatLngToDivPixel(centerCurrent);
            // Update the canvas CSS position and dimensions.
            this.UpdatePosition(cc.x - newWidth / 2, cc.y - newHeight / 2, newWidth, newHeight);
        }
    };
    /**
     * CanvasOverlay loaded, attach map events for updating canvas.
     * @method
     * @public
     * @memberof GoogleCanvasOverlay
     */
    GoogleCanvasOverlay.prototype.OnLoad = function () {
        var _this = this;
        var isStreetView = false;
        var map = this.getMap();
        // Get the current map view information.
        this._zoomStart = map.getZoom();
        var c = map.getCenter();
        this._centerStart = {
            latitude: c.lat(),
            longitude: c.lng()
        };
        // When the map stops moving, render new data on the canvas.
        this._viewChangeEndEvent = google.maps.event.addListener(map, 'idle', function (e) {
            _this.UpdateCanvas();
        });
        // Update the position of the overlay when the map is resized.
        this._mapResizeEvent = google.maps.event.addListener(map, 'resize', function (e) {
            _this.UpdateCanvas();
        });
    };
    /**
     * Associates the cnavas overlay with a map.
     * @method
     * @public
     * @memberof GoogleCanvasOverlay
     */
    GoogleCanvasOverlay.prototype.SetMap = function (map) {
        this.setMap(map);
    };
    ///
    /// Protected methods
    ///
    /**
     * Attaches the canvas to the map.
     * @memberof CanvasOverlay
     * @method
     * @public
     */
    GoogleCanvasOverlay.prototype.SetCanvasElement = function (el) {
        var panes = this.getPanes();
        if (panes) {
            if (el != null) {
                panes.overlayLayer.appendChild(el);
                // 4: floatPane (infowindow)
                // 3: overlayMouseTarget (mouse events)
                // 2: markerLayer (marker images)
                // 1: overlayLayer (polygons, polylines, ground overlays, tile layer overlays)
                // 0: mapPane (lowest pane above the map tiles)
            }
            else {
                panes.overlayLayer.removeChild(this._canvas);
            }
        }
    };
    /**
     * Remove the map event handlers.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    GoogleCanvasOverlay.prototype.RemoveEventHandlers = function () {
        // Remove all event handlers from the map.
        if (this._viewChangeEndEvent) {
            google.maps.event.removeListener(this._viewChangeEndEvent);
        }
        if (this._mapResizeEvent) {
            google.maps.event.removeListener(this._mapResizeEvent);
        }
    };
    /**
     * Updates the Canvas size based on the map size.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    GoogleCanvasOverlay.prototype.Resize = function () {
        var map = this.getMap();
        // Clear canvas by updating dimensions. This also ensures canvas stays the same size as the map.
        var el = map.getDiv();
        this._canvas.width = el.offsetWidth;
        this._canvas.height = el.offsetHeight;
    };
    /**
     * Updates the Canvas.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    GoogleCanvasOverlay.prototype.UpdateCanvas = function () {
        var map = this.getMap();
        // Only render the canvas if it isn't in streetside mode.
        if (true) {
            this._canvas.style.display = '';
            // Reset CSS position and dimensions of canvas.
            var el = map.getDiv();
            var w = el.offsetWidth;
            var h = el.offsetHeight;
            var centerPoint = this.getProjection().fromLatLngToDivPixel(map.getCenter());
            this.UpdatePosition((centerPoint.x - w / 2), (centerPoint.y - h / 2), w, h);
            // Redraw the canvas.
            this.Redraw(true);
            // Get the current map view information.
            this._zoomStart = map.getZoom();
            var c = map.getCenter();
            this._centerStart = {
                latitude: c.lat(),
                longitude: c.lng()
            };
        }
    };
    return GoogleCanvasOverlay;
}(canvas_overlay_1.CanvasOverlay));
exports.GoogleCanvasOverlay = GoogleCanvasOverlay;
/**
 * Helper function to extend the OverlayView into the CanvasOverlay
 *
 * @export
 * @method
 */
function MixinCanvasOverlay() {
    var x = GoogleCanvasOverlay.prototype;
    GoogleCanvasOverlay.prototype = new google.maps.OverlayView();
    for (var y in x) {
        if (x[y] != null) {
            GoogleCanvasOverlay.prototype[y] = x[y];
        }
    }
    GoogleCanvasOverlay.prototype['onAdd'] = x['OnAdd'];
    GoogleCanvasOverlay.prototype['draw'] = x['OnDraw'];
    GoogleCanvasOverlay.prototype['onRemove'] = x['OnRemove'];
}
exports.MixinCanvasOverlay = MixinCanvasOverlay;
//# sourceMappingURL=google-canvas-overlay.js.map