import { IInfoWindowOptions } from '../../interfaces/iinfo-window-options';
import { ILatLong } from '../../interfaces/ilatlong';
import { GoogleMapService } from '../../services/google/google-map.service';
import { InfoWindow } from '../info-window';
import * as GoogleMapTypes from '../../services/google/google-map-types';
/**
 * Concrete implementation for a {@link InfoWindow}} model for Google Maps.
 *
 * @implements InfoWindow
 * @class GoogleInfoWindow
 * @export
 */
export declare class GoogleInfoWindow implements InfoWindow {
    private _infoWindow;
    private _mapService;
    private _isOpen;
    /**
     * Gets whether the info box is currently open.
     *
     * @readonly
     * @type {boolean}
     * @memberof InfoWGoogleInfoWindowindow
     */
    readonly IsOpen: boolean;
    /**
     * Gets the underlying native object.
     *
     * @type {GoogleMapTypes.InfoWindow}
     * @property
     * @public
     * @readonly
     */
    readonly NativePrimitve: GoogleMapTypes.InfoWindow;
    /**
     * Creates an instance of GoogleInfoWindow.
     * @param {GoogleMapTypes.InfoWindow} _infoWindow - A {@link GoogleMapTypes.InfoWindow} instance underlying the model.
     * @param {GoogleMapService} _mapService - An instance of the {@link GoogleMapService}.
     * @memberof GoogleInfoWindow
     * @public
     * @constructor
     */
    constructor(_infoWindow: GoogleMapTypes.InfoWindow, _mapService: GoogleMapService);
    /**
      * Adds an event listener to the InfoWindow.
      *
      * @param {string} eventType - String containing the event for which to register the listener (e.g. "click")
      * @param {Function} fn - Delegate invoked when the event occurs.
      *
      * @memberof GoogleInfoWindow
      * @method
      * @public
      */
    AddListener(eventType: string, fn: Function): void;
    /**
     *
     * Closes the info window.
     *
     * @memberof GoogleInfoWindow
     * @method
     * @public
     */
    Close(): void;
    /**
     * Gets the position of the info window
     *
     * @returns {ILatLong} - The geo coordinates of the info window.
     *
     * @memberof GoogleInfoWindow
     * @method
     * @public
     */
    GetPosition(): ILatLong;
    /**
     * Opens the info window
     *
     * @param {*} [anchor] - Optional Anchor.
     *
     * @memberof GoogleInfoWindow
     * @method
     * @public
     */
    Open(anchor?: any): void;
    /**
     * Sets the info window options
     *
     * @param {IInfoWindowOptions} options - The options to set. This object will be merged with the existing options.
     *
     * @memberof GoogleInfoWindow
     * @method
     * @public
     */
    SetOptions(options: IInfoWindowOptions): void;
    /**
     * Sets the info window position
     *
     * @param {ILatLong} position - Geo coordinates at which to anchor the info window.
     *
     * @memberof GoogleInfoWindow
     * @method
     * @public
     */
    SetPosition(position: ILatLong): void;
}
