"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var google_layer_1 = require("../../models/google/google-layer");
var google_polygon_1 = require("../../models/google/google-polygon");
var google_polyline_1 = require("../../models/google/google-polyline");
var google_layer_base_1 = require("./google-layer-base");
var map_service_1 = require("../map.service");
var google_conversions_1 = require("./google-conversions");
/**
 * Implements the {@link LayerService} contract for a Google Maps specific implementation.
 *
 * @export
 * @class GoogleLayerService
 * @extends {GoogleLayerBase}
 * @implements {LayerService}
 */
var GoogleLayerService = /** @class */ (function (_super) {
    __extends(GoogleLayerService, _super);
    ///
    /// Constructor
    ///
    /**
     * Creates an instance of GoogleLayerService.
     * @param {MapService} _mapService - Instance of the Google Maps Service. Will generally be injected.
     * @param {NgZone} _zone - NgZone instance to provide zone aware promises.
     *
     * @memberof GoogleLayerService
     */
    function GoogleLayerService(_mapService, _zone) {
        var _this = _super.call(this, _mapService) || this;
        _this._zone = _zone;
        ///
        /// Field Declarations.
        ///
        _this._layers = new Map();
        return _this;
    }
    /**
     * Adds a layer to the map.
     *
     * @abstract
     * @param {MapLayerDirective} layer - MapLayerDirective component object.
     * Generally, MapLayerDirective will be injected with an instance of the
     * LayerService and then self register on initialization.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.AddLayer = function (layer) {
        var _this = this;
        var p = new Promise(function (resolve, reject) {
            _this._mapService.MapPromise.then(function (m) {
                resolve(new google_layer_1.GoogleLayer(m, _this._mapService, layer.Id));
            });
        });
        this._layers.set(layer.Id, p);
    };
    ;
    /**
     * Returns the Layer model represented by this layer.
     *
     * @abstract
     * @param {MapLayerDirective} layer - MapLayerDirective component object for which to retrieve the layer model.
     * @returns {Promise<Layer>} - A promise that when resolved contains the Layer model.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.GetNativeLayer = function (layer) {
        return this._layers.get(layer.Id);
    };
    ;
    /**
     * Deletes the layer
     *
     * @abstract
     * @param {MapLayerDirective} layer - MapLayerDirective component object for which to retrieve the layer.
     * @returns {Promise<void>} - A promise that is fullfilled when the layer has been removed.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.DeleteLayer = function (layer) {
        var _this = this;
        var l = this._layers.get(layer.Id);
        if (l == null) {
            return Promise.resolve();
        }
        return l.then(function (l1) {
            return _this._zone.run(function () {
                l1.Delete();
                _this._layers.delete(layer.Id);
            });
        });
    };
    ;
    /**
     * Adds a polygon to the layer.
     *
     * @abstract
     * @param {number} layer - The id of the layer to which to add the polygon.
     * @param {IPolygonOptions} options - Polygon options defining the polygon.
     * @returns {Promise<Polygon>} - A promise that when fullfilled contains the an instance of the Polygon model.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.CreatePolygon = function (layer, options) {
        var p = this._mapService.CreatePolygon(options);
        var l = this._layers.get(layer);
        Promise.all([p, l]).then(function (x) { return x[1].AddEntity(x[0]); });
        return p;
    };
    ;
    /**
     * Creates an array of unbound polygons. Use this method to create arrays of polygons to be used in bulk
     * operations.
     *
     * @param {number} layer - The id of the layer to which to add the polygon.
     * @param {Array<IPolygonOptions>} options - Polygon options defining the polygons.
     * @returns {Promise<Array<Polygon>>} - A promise that when fullfilled contains the an arrays of the Polygon models.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.CreatePolygons = function (layer, options) {
        //
        // Note: we attempted using data.Polygons in an attempt to improve performance, but either data.Polygon
        // or data.MultiPolygon actually operate significantly slower than generating the polygons this way.
        // the slowness in google as opposed to bing probably comes from the point reduction algorithm uses.
        // Signigicant performance improvements might be possible in google when using a pixel based reduction algorithm
        // prior to setting the polygon path. This will lower to processing overhead of the google algorithm (with is Douglas-Peucker
        // and rather compute intensive)
        //
        var p = this.GetLayerById(layer);
        if (p == null) {
            throw (new Error("Layer with id " + layer + " not found in Layer Map"));
        }
        return p.then(function (l) {
            var polygons = new Promise(function (resolve, reject) {
                var polys = options.map(function (o) {
                    var op = google_conversions_1.GoogleConversions.TranslatePolygonOptions(o);
                    var poly = new google.maps.Polygon(op);
                    var polygon = new google_polygon_1.GooglePolygon(poly);
                    if (o.title && o.title !== '') {
                        polygon.Title = o.title;
                    }
                    if (o.metadata) {
                        o.metadata.forEach(function (val, key) { return polygon.Metadata.set(key, val); });
                    }
                    return polygon;
                });
                resolve(polys);
            });
            return polygons;
        });
    };
    /**
     * Adds a polyline to the layer.
     *
     * @abstract
     * @param {number} layer - The id of the layer to which to add the polyline.
     * @param {IPolylineOptions} options - Polyline options defining the polyline.
     * @returns {Promise<Polyline|Array<Polyline>>} - A promise that when fullfilled contains the an instance of the Polyline (or an array
     * of polygons for complex paths) model.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.CreatePolyline = function (layer, options) {
        var p = this._mapService.CreatePolyline(options);
        var l = this._layers.get(layer);
        Promise.all([p, l]).then(function (x) {
            var p1 = Array.isArray(x[0]) ? x[0] : [x[0]];
            for (var _i = 0, p1_1 = p1; _i < p1_1.length; _i++) {
                var p2 = p1_1[_i];
                x[1].AddEntity(p2);
            }
        });
        return p;
    };
    ;
    /**
     * Creates an array of unbound polylines. Use this method to create arrays of polylines to be used in bulk
     * operations.
     *
     * @param {number} layer - The id of the layer to which to add the polylines.
     * @param {Array<IPolylineOptions>} options - Polyline options defining the polylines.
     * @returns {Promise<Array<Polyline|Array<Polyline>>>} - A promise that when fullfilled contains the an arrays of the Polyline models.
     *
     * @memberof GoogleLayerService
     */
    GoogleLayerService.prototype.CreatePolylines = function (layer, options) {
        var p = this.GetLayerById(layer);
        if (p == null) {
            throw (new Error("Layer with id " + layer + " not found in Layer Map"));
        }
        return p.then(function (l) {
            var polylines = new Promise(function (resolve, reject) {
                var polys = options.map(function (o) {
                    var op = google_conversions_1.GoogleConversions.TranslatePolylineOptions(o);
                    if (o.path && o.path.length > 0 && !Array.isArray(o.path[0])) {
                        op.path = google_conversions_1.GoogleConversions.TranslatePaths(o.path)[0];
                        var poly = new google.maps.Polyline(op);
                        var polyline_1 = new google_polyline_1.GooglePolyline(poly);
                        if (o.title && o.title !== '') {
                            polyline_1.Title = o.title;
                        }
                        if (o.metadata) {
                            o.metadata.forEach(function (v, k) { return polyline_1.Metadata.set(k, v); });
                        }
                        return polyline_1;
                    }
                    else {
                        var paths = google_conversions_1.GoogleConversions.TranslatePaths(o.path);
                        var lines_1 = new Array();
                        paths.forEach(function (x) {
                            op.path = x;
                            var poly = new google.maps.Polyline(op);
                            var polyline = new google_polyline_1.GooglePolyline(poly);
                            if (o.metadata) {
                                o.metadata.forEach(function (v, k) { return polyline.Metadata.set(k, v); });
                            }
                            if (o.title && o.title !== '') {
                                polyline.Title = o.title;
                            }
                            lines_1.push(polyline);
                        });
                        return lines_1;
                    }
                });
                resolve(polys);
            });
            return polylines;
        });
    };
    GoogleLayerService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    GoogleLayerService.ctorParameters = function () { return [
        { type: map_service_1.MapService, },
        { type: core_1.NgZone, },
    ]; };
    return GoogleLayerService;
}(google_layer_base_1.GoogleLayerBase));
exports.GoogleLayerService = GoogleLayerService;
//# sourceMappingURL=google-layer.service.js.map