"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
/**
 * Abstract class to implement map api. A concrete implementation should be created for each
 * Map provider supported (e.g. Bing, Goolge, ESRI)
 *
 * @export
 * @abstract
 * @class MapService
 */
var MapService = /** @class */ (function () {
    function MapService() {
    }
    ///
    /// Public methods and MapService interface implementation
    ///
    /**
     * Gets a random geo locations filling the bounding box.
     *
     * @static
     * @param {number} count - number of locations to return
     * @param {IBox} bounds  - bounding box.
     * @returns {Array<ILatLong>} - Array of geo locations.
     * @memberof MapService
     */
    MapService.GetRandonLocations = function (count, bounds) {
        var a = [];
        var _getRandomLocation = function (b) {
            var lat = Math.random() * (b.maxLatitude - b.minLatitude) + b.minLatitude;
            var lng = 0;
            if (crossesDateLine) {
                lng = Math.random() * (b.minLongitude + 360 - b.maxLongitude) + b.maxLongitude;
                if (lng > 180) {
                    lng = lng - 360;
                }
            }
            else {
                lng = Math.random() * (b.maxLongitude - b.minLongitude) + b.minLongitude;
            }
            var p = { latitude: lat, longitude: lng };
            return p;
        };
        var crossesDateLine = false;
        if (bounds == null) {
            bounds = {
                maxLatitude: 360,
                minLatitude: 0,
                maxLongitude: 170,
                minLongitude: 0
            };
        }
        if (bounds.center.longitude < bounds.minLongitude || bounds.center.longitude > bounds.maxLongitude) {
            crossesDateLine = true;
        }
        if (!count || count <= 0) {
            return [_getRandomLocation(bounds)];
        }
        for (var r = 0; r < count; r++) {
            a.push(_getRandomLocation(bounds));
        }
        return a;
    };
    MapService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    MapService.ctorParameters = function () { return []; };
    return MapService;
}());
exports.MapService = MapService;
//# sourceMappingURL=map.service.js.map